//
//  preprocessor.cpp
//  Lily
//
//  Created by KMY on 2015/05/09.
//  Copyright (c) 2015年 KMY. All rights reserved.
//

#include "preprocessor.h"
#include "script.h"
#include "token.h"
#include "macro.h"
#include "message.h"
#include "generate.h"

PreProcessor::PreProcessor(Script* script) {
	scriptObj = script;
}

void PreProcessor::start() {
	
	// トークンを取得
	const lchar* tokenList = scriptObj->getTokenLchar();
	const lchar* tokenType = tokenList;
	const lchar* tokenValue = tokenList+1;
	const lchar* beforeTokenType = 0;

	int indent = 0;
	bool isAnyName = false;
	bool isMacroName = false;				// マクロ名を処理中であるか
	int macroIndent = 0;					// 現在処理中マクロのインデント数
	
	Macro* macro = 0;
	const lchar* paramName = 0;				// 引数名であると思われるトークン
	int paramType = 0;						// 型名であると思われるトークン
	const lchar* paramTypeName = 0;			// 型名（文字列直接格納）と思われるトークン
	
	// 行番号リセット
	scriptObj->setLineNumber(1);
	
	// 反復
	while (*tokenType != Token::TOKEN_ENDFILE) {
		
		//Token::tokenTest(tokenType);
		
		// インデントを確認
		if (*tokenType == Token::TOKEN_INDENT ||
			(beforeTokenType && *beforeTokenType == Token::TOKEN_YN)) {
			
			// インデントなしであるか？
			if (*tokenType != Token::TOKEN_INDENT) {
				indent = 0;
			}
			else {
				// インデント更新
				indent = *tokenValue;
			}
		}

		
		// 関数やマクロ名の処理
		if (isAnyName) {
			
			// 改行で終了
			if (*tokenType == Token::TOKEN_YN) {
				
				// マクロを登録
				if (isMacroName) {
					macro->setName(beforeTokenType+1);
					macro->setType(Variable::VT_VOID);
					macro->setStartToken(Token::nextToken(tokenType));
					Generate::addGlobalMacro(macro);
					isMacroName = false;
					macro = 0;
				}
				
				isAnyName = false;
			}
			
			// マクロ処理
			else if (isMacroName) {
				
				// マクロオブジェクトの生成
				if (*beforeTokenType == Token::TOKEN_S_MACRO) {
					if (macro) {
						Message::error("マクロの定義が不正です", Message::MES_CORE_ERROR, scriptObj);
						//delete macro;
						isAnyName = isMacroName = false;
					}
					else {
						macro = new Macro();
					}
				}
				
				// 引数型か引数名か判断
				if (*tokenType == Token::TOKEN_UNKNOWN) {
					
					// 引数型
					if (*beforeTokenType == Token::TOKEN_SYMBOL && *(beforeTokenType+1) == Token::SYMBOL_MKAKKOS) {
						if (paramType) {
							Message::error("マクロ引数の定義が不完全です", Message::MES_CORE_ERROR, scriptObj);
							delete macro;
							isAnyName = isMacroName = false;
						}
						// システム定義であるか？
						int pt = Variable::getLilyType(tokenValue);
						if (pt == Variable::VT_UNKNOWN)
							paramTypeName = tokenValue;
						else
							paramType = pt;
					}
					
					// 引数名
					else if (*beforeTokenType == Token::TOKEN_JOSHI ||
							 *beforeTokenType == Token::TOKEN_S_MACRO) {
						if (paramName || paramTypeName) {
							Message::error("マクロ引数の定義が不完全です", Message::MES_CORE_ERROR, scriptObj);
							delete macro;
							isAnyName = isMacroName = false;
						}
						paramName = tokenValue;
					}
				}
				
				// 型指定の終わり：引数オブジェクトを生成
				else if (*tokenType == Token::TOKEN_JOSHI) {
					if (!paramName || (!paramType && !paramTypeName)) {
						Message::error("マクロ引数の定義が不完全です", Message::MES_CORE_ERROR, scriptObj);
						delete macro;
						isAnyName = isMacroName = false;
					}
					else {
						Variable* newParam;
						if (paramType)
							newParam = new Variable(paramName, paramType);
						else
							newParam = new Variable(paramName, paramType);
						newParam->setJoshi(*tokenValue);
						macro->addParam(newParam);
						paramName = 0;
						paramType = 0;
						paramTypeName = 0;
					}
				}
			}
		}
		
		// グローバルスコープならば開始
		else if (!indent) {
			
			// マクロ処理を開始
			if (*tokenType == Token::TOKEN_S_MACRO) {
				isAnyName = isMacroName = true;
				paramName = 0;
				paramType = 0;
				paramTypeName = 0;
			}
		}
		
		// 改行なら改行
		if (*tokenType == Token::TOKEN_YN) {
			scriptObj->nextLine();
		}
		 
		// 次のトークンへ
		beforeTokenType = tokenType;
		tokenType = Token::nextToken(tokenType);
		tokenValue = tokenType + 1;
	}
	
}
