//
//  variable.h
//  Lily
//
//  Created by KMY on 2015/05/09.
//  Copyright (c) 2015年 KMY. All rights reserved.
//

#ifndef __Lily__variable__
#define __Lily__variable__

#include <stdio.h>
#include "character.h"

class Variable {
public:
	// ユーザ定義の型は0より大きい。value値がそのままTypeになる
	enum VariableType {
		VT_VOID=-1,
		VT_UNKNOWN=-2,
		VT_INT=-3,				// 32ビット符号付き数値
		VT_STR=-4,				// Unicode文字列（半角文字も2バイトになるあれ）
		VT_USER = -100,			// ユーザ定義の型の可能性
	};
public:
	Variable();
	Variable(const lchar* vn, int vt, bool literalFlagp = false);
	Variable(const lchar* vn, const lchar* vtn);
	virtual ~Variable();
	Variable* clone();
	void setArrayFlag(bool v);
	bool getArrayFlag();
	bool getLiteralFlag();
	Variable* next();
	Variable* add(const lchar* vn, int vt);
	Variable* add(const lchar* vn, const lchar* vtn);
	Variable* add(Variable* obj);
	Variable* getLast();
	bool checkVariable(int vt, const lchar* vn);
	bool checkVariable(Variable* obj, bool nameCheckFlag = true);
	void setJoshi(int j);
	char* getJavaName();						// Javaソースに直接埋め込む名前を取得
	char* getJavaType();						// Javaソースに直接埋め込む変数型を取得
	int getJoshi();
	int getValue();
	int getType();
	static int getLilyType(const lchar* typeName);
	void setType(int vt);
	const lchar* getLiteral();
	void setLiteral(const lchar* l);
	const lchar* getName();
	void setName(const lchar* vn);
	void setEveryName(Variable* obj);					// nextも含めて名前部分だけコピー（関数の仮引数のため）
	void variableTest();
private:
	void init();
	Variable* searchLast();
protected:
	lchar* name;				// Lilyにおける変数名
	int value;					// javaにおける変数名の連番
	int type;					// 変数関数の型
	lchar* typeName;			// ユーザ定義の型名
	int joshi;					// 助詞の種類（関数の引数についての処理で使う）
	const lchar* literal;		// 数値／文字列リテラル（引数処理などで使う）
	Variable* nextObj;			// 次のオブジェクト
	Variable* lastObj;			// 最後のオブジェクト（正しいデータを保有するのは先頭のオブジェクトのみ）
	bool arrayFlag;
	bool literalFlag;
private:
	static int count;
};

#endif /* defined(__Lily__variable__) */
